// ---------------------------------------------------------------------------
// Created by F. Malpartida on 9/9/12.
// Copyright 2011 - Under creative commons license 3.0:
//        Attribution-NonCommercial-ShareAlike CC BY-NC-SA
//        http://www.electrofunltd.com
//
// This software is furnished "as is", without technical support, and with no
// warranty, express or implied, as to its usefulness for any purpose.
//
// Thread Safe: No
// Extendable: No
//
// @file processGcode.cpp
// Main Gcode parser entry point.
//
// @brief CNC gcode interpreter for 3 axis CNC machine. Configured for ELFSHL1
//     3 axis stepper motor driver shield.
//
// Acknowledgements:
// Inspired on the work of Carlos guilarte y diego colonnello (Txapuzas CNC).
//
// @author F. Malpartida
// ---------------------------------------------------------------------------
#include <Arduino.h>
#include "configMe.h"
#include "stepperCntrl.h"
#include "processGcode.h"

// PRIVATE CONSTANT DECLARATION
// ---------------------------------------------------------------------------
char         inBuffer[COMMAND_SIZE]; // input buffer for serial input
uint16_t     serial_count;           // number of bytes read from serial line
stepperCntrl *myController = NULL;   // machine controller that will execute gcodes
#ifdef _TEMP_CONTROL_ENABLED_
tempCntrl    *myTempController = NULL;
#endif

// PRIVATE FUNCTIONS
// ---------------------------------------------------------------------------

/*!
 @function
 @abstract   Initializes internal data structures
 @discussion Initializes internal data structures and sets the to 0.
 */
static void initBuffer (void)
{
   memset (inBuffer, 0, sizeof(inBuffer) );
   serial_count = 0;
}

// PUBLIC FUNCTIONS
// ---------------------------------------------------------------------------
//init our string processing
#ifdef _TEMP_CONTROL_ENABLED_
void initGcodeProcessor( stepperCntrl *iController, tempCntrl *iTempCntrl )
{
   //init our command buffer
   initBuffer ( );
   myController = iController;
   myTempController = iTempCntrl;
}
#else
void initGcodeProcessor( stepperCntrl *iController )
{
   //init our command
   initBuffer ( );
   myController = iController;   
}
#endif

// processGcode
void processGcode ( void )
{
   char       c = 0;
   static int no_data = 0;
   
   // read in characters if they are available on serial line.
   // --------------------------------------------------------
   if (Serial.available() > 0)
   {
      c = Serial.read();
      no_data = 0;
      
      // newlines are ends of commands
      // -----------------------------
      if (c != '\n')
      {
         inBuffer[serial_count] = c;
         serial_count++;
      }
   }
   //mark no data
   else
   {
      no_data++;
      delayMicroseconds(100);
   }
   
   // If there is a pause or we got a real command, do it
   // ---------------------------------------------------
   if ((serial_count > 0) && (c == '\n' || no_data > 100))
   {
      //process our command!
      processString(inBuffer, serial_count);
      // clear the buffer
      initBuffer();
   }
   
   
   // If there is no data available after 1 second turn off steppers
   // The controller is set to standby mode
   // --------------------------------------------------------------
   if (no_data > 10000)
   {
      myController->disableSteppers();
   }
}



//Read the string and execute instructions
static void processString(char instruction[], int size)
{
   stepperCntrl::t_floatPoint  fp;
   uint8_t code = 0;
   
   fp.x = 0.0;
   fp.y = 0.0;
   fp.z = 0.0;
   
   // the character "/" is used to insert comments in the gcode
   // ---------------------------------------------------------
   if (instruction[0] == '/')
   {
      Serial.println("ok");
      return;
   }
   
   // Verify that we have a G code
   // ----------------------------
   if (
       has_command('G', instruction, size) ||
       has_command('X', instruction, size) ||
       has_command('Y', instruction, size) ||
       has_command('Z', instruction, size)
       )
   {
      // Extract the G code
      code = (int)search_string('G', instruction, size);
      
      // Get co-ordinates if required by the code type given
      // ---------------------------------------------------
      switch (code)
      {
         case 0:
            // No G code code received as such, only coordinates
         case 1:
         case 2:
         case 3:
         case 30:
            if ( has_command('F', instruction, size) )
            {
               myController->setFeedRate(search_string('F', instruction, size));
            }
            
            // If in absolute coordinates get the current position and get the
            // new coordinates
            // ---------------------------------------------------------------
            if ( myController->absoluteCoord () )
            {
               fp = myController->getPosition();
            }
            
            // Extract the new coordinates received to get the new position
            // the controller should handle the coordinate reference.
            // --------------------------------------------------------------
            if ( has_command('X', instruction, size) )
               fp.x = search_string('X', instruction, size);
            if ( has_command('Y', instruction, size) )
               fp.y = search_string('Y', instruction, size);
            if ( has_command('Z', instruction, size) )
               fp.z = search_string('Z', instruction, size);
            
            break;
      }
      
      // Execute the actions associated to the G code
      // --------------------------------------------
      switch (code)
      {
            // Rapid Positioning command and linear interpolation
            // these are basically the same thing.
         case 0:
         case 1:
            // set our target.
            myController->setTarget(fp.x, fp.y, fp.z);
            
            // move to specified target
            myController->moveLine();
            break;
            
            //Clockwise arc
         case 2:
            //Counterclockwise arc
         case 3:
            myController->moveArc (fp, search_string('I', instruction, size),
                                   search_string('J', instruction, size),
                                   (code == 2));
            break;
            
            // Dwell - pause
         case 4:
            delay((int)search_string('P', instruction, size));
            break;
            
            // Set inches for base control units
         case 20:
            myController->setUnits(MM_PER_INCH);
            myController->setCurveSection(CURVE_SECTION_INCHES);
            myController->computeDeltas();
            break;
            
            // Set mm for base control units
         case 21:
            myController->setUnits(MM);
            myController->setCurveSection(CURVE_SECTION_MM);
            myController->computeDeltas();
            break;
            
            // go home via an intermediate point.
         case 30:
            myController->setTarget(fp.x, fp.y, fp.z);
            myController->moveLine();
            
            // go home.
         case 28:
            myController->setTarget(0.0, 0.0, 0.0);
            myController->moveLine();
            break;
            
            // Absolute Positioning
         case 90:
            myController->setAbsoluteCoord();
            break;
            
            // Incremental Positioning
         case 91:
            myController->setRelativeCoord();
            break;
            
            // Set current possiton as home
         case 92:
            myController->setPosition(0.0, 0.0, 0.0);
            break;
            
            /*
             //Inverse Time Feed Mode
             case 93:
             
             break;  //TODO: add this
             
             //Feed per Minute Mode
             case 94:
             
             break;  //TODO: add this
             */
            
         default:
            Serial.print("error ");
            Serial.println(code,DEC);
      }
   }
   
   // Verify if the command is an M code
   // -------------------------------------
   if (has_command('M', instruction, size))
   {
      code = search_string('M', instruction, size);
      switch (code)
      {
         case 0:
            break;
            
            // motor ON
         case 3:
            Serial.println("CNC:Motor ON");
            digitalWrite(MOTOR_PIN, HIGH);
            break;
            
            // motor OFF
         case 5:
            Serial.println("CNC:Motor OFF");
            digitalWrite (MOTOR_PIN, LOW);
            break;
            
            // Set absolute coordinates for E axis (currently not supported)
         case 82:
            break;
            
            // Set relative coordinates for E axis (currently not supported)
         case 83:
            break;
            
            // stop idle hold
         case 84:
            myController->disableSteppers();
            break;
            
         case 105:
            Serial.print("T:");
#ifdef _TEMP_CONTROL_ENABLED_
            Serial.println (myTempController->tempGet(),2);
#else
            Serial.println ( 0.0,2 );
#endif
            break;
            
            //case 114:
            //   Serial.println("C:0.0 0.0");
            //   break;
            
            // Switch fan ON - currently not supported, everything is in automatic
         case 7:
			case 106:
            break;
            
            // Switch fan OFF - currently not supported
         case 8:
         case 107:
            break;
            
				// M110: Set Current Line Number
				//
				// Example: N123 M110
				//
				// Set the current line number to 123.  Thus the expected next line
            // after this command will be 124. This is a NOP.
				//
         case 110:
				break;
            
            // Get current position C: X:<pos x> Y:<pos y> Z:<pos z>
         case 114:
            fp = myController->getPosition();
            Serial.print ("C: X:");
            Serial.print ( fp.x, 4 );
            Serial.print ( " Y:" );
            Serial.print ( fp.y, 4 );
            Serial.print ( " Z:");
            Serial.println ( fp.z, 4 );
            break;
            
            // Get the firmware version
         case 115:
            Serial.print (F("FIRMWARE_NAME:ELFCNC FIRMWARE_URL:https://bitbucket.org/fmalpartida/cnc-controller VERSION: "));
            Serial.println( FRMW_VERSION );
            break;
            
				// M200: report endstop status
				// Report the current status of the endstops if available.
            // Currently not supported
         case 200:
            break;
            
         default:
            Serial.print("error ");
            Serial.println(code, DEC);
      }
   }
   
   // report back execution
   Serial.println("ok");
}

// look for the number that appears after the char key and return it
static double search_string(char key, char instruction[], int string_size)
{
   char temp[10] = "";
   
   // search in the string the value key
   for (byte i=0; i < string_size; i++)
   {
      // if the key has been found in the input buffer
      // get its associated value
      // ---------------------------------------------
      if (instruction[i] == key)
      {
         i++;
         int k = 0;
         while (i < string_size && k < 10)
         {
            // if null or ' ', stop
            if (instruction[i] == 0 || instruction[i] == ' ')
               break;
            
            // build up the string with the value
            temp[k] = instruction[i];
            i++;
            k++;
         }
         // convert the string to value to a double
         return strtod(temp, NULL);
      }
   }
   return 0;
}

//look for the command if it exists.
static bool has_command(char key, char instruction[], int string_size)
{
   // Search for the key through the entire string, return as soon as its found.
   // --------------------------------------------------------------------------
   for (byte i=0; i<string_size; i++)
   {
      if (instruction[i] == key)
         return true;
   }
   return false;
}

