// ---------------------------------------------------------------------------
// Created by Francisco Malpartida on 16/04/11.
// Copyright 2011 - Under creative commons license 3.0:
//        Attribution-NonCommercial-ShareAlike CC BY-NC-SA
//        http://www.electrofunltd.com
//
// This software is furnished "as is", without technical support, and with no 
// warranty, express or implied, as to its usefulness for any purpose.
//
// Thread Safe: No
// Extendable: No
//
// @file stepper.h
// Stepper driver control class. This class is capable of driver stepper motors
// using step and direction signals such as: ELFDRV1, ELFDRV2, stepper stick,
// those developed by pololu, etc.
// 
// @brief Stepper driver
//
// @author F. Malpartida
// ---------------------------------------------------------------------------
#ifndef _STEPPER_H_
#define _STEPPER_H_
#include <stdint.h>
#include "configMe.h"

// If the MMI was enabled use the FastIO from the library
#ifdef _MMI_ENABLED_
#include <FastIO.h>
#else
#include "FastIO.h"
#endif



// Stepper class definition
// ------------------------
class stepper
{
public:
   typedef enum { CW = 0, ACW = 1 } t_dir;
   
   /*!
    @function
    @abstract   Class constructor
    @discussion Create a stepper montor controller object associated to
    a step control pin, direction control pin, an enable pin. The prototype
    also allows to control the step pulse width. 5 us seams as a good over all
    value, althought most drivers work well with 1us step pulse.
    If the enPin is set to 0, it indicates that there is no pin attached to
    control the the enable line of the stepper driver.
    @param      stepPin: step control pin
    @param      dirPin:  direction contrl pin
    @param      enPin:   enable control pin. 0 indicates that no enable pin is
    used.
    @param      pulseW: pulse witdh on the step pin, specified in micro-seconds.
    */
   stepper ( uint8_t stepPin, uint8_t dirPin, uint8_t enPin, uint8_t pulseW );

   /*!
    @function
    @abstract   Step the driver by one.
    @discussion Step the driver one pulse using the currently set pulse width and
    direction.
    */   
   void     step ( void );
   
   /*!
    @function
    @abstract   Steps one step if the step counter is different to zero. 
    @discussion Steps one step if the step counter is not zero (0) and decrements
    the number of steps left by one. This method is used in conjunction with
    the setSteps to schedule the execution of a number of steps. The method
    has to be called continuosly for the motor to step and will stop controlling
    the step pulse once the zero counter is 0.
    */      
   void     runStep ( void );

   /*!
    @function
    @abstract   Plays back all the steps configured by setSteps.
    @discussion Steps through the whole secuence of steps configured using the
    setSteps method. This method is used in conjunction with
    the setSteps to schedule the execution of a number of steps. The method
    has to be called continuosly for the motor to step and will stop controlling
    the step pulse once the zero counter is 0.
    */
   void play ( void );
   
   /*!
    @function
    @abstract   sets the motor rotation direction.
    @discussion Set the motor step direction (CW or ACW).
    @param      direction: CW or ACW.
    */         
   void     setDirection ( t_dir direction );
   
   /*!
    @function
    @abstract   Configure the number steps to pulse.
    @discussion Configures the number of steps to pulse the "step" pin. 
    @param      numSteps: number of steps to configure the driver.
    */         
   void     setSteps ( uint16_t numSteps );
   
   /*!
    @function
    @abstract   Enable the stepper driver.
    @discussion Enables the stepper driver if the configured control pin is 
    different than 0.
    */         
   void     enable ( void );
   
   /*!
    @function
    @abstract   Disables the stepper driver.
    @discussion Disables the stepper driver if the configured control pin is 
    different than 0.
    */         
   void     disable ( void );

   /*!
    @function
    @abstract   Returns the number of steps remaining.
    @discussion Returns the number of steps remaining to take after calling the
    setSteps.
    @result     Number of remaining steps to schedule.
    */         
   uint16_t getSteps ( void );
   
   
private:
   fio_bit       _stepPin;   // digital IO used to drive the step pin
   fio_register  _stepReg;   // Step pin register
   fio_bit       _dirPin;    // digital IO used to configure the direction pin
   fio_register  _dirReg;    // direction pin register
   uint8_t       _enPin;     // digital IO used to enable the driver
   uint8_t       _pulseW;    // pulse width in useconds
   uint16_t      _numSteps;  // number of steps to take when calling the runStep method
   
};

#endif
