// ---------------------------------------------------------------------------
// Created by F. Malpartida on 9/9/12.
// Copyright 2011 - Under creative commons license 3.0:
//        Attribution-NonCommercial-ShareAlike CC BY-NC-SA
//        http://www.electrofunltd.com
// 
// This software is furnished "as is", without technical support, and with no 
// warranty, express or implied, as to its usefulness for any purpose.
//
// Thread Safe: No
// Extendable: No
//
// @file tempCntrl.cpp
// Main temperature controller and fan interface. Senses temperature from LM75B
// and controls the fan speed to cool the electronics. 
// Requires thermistor library.
// 
// @brief 
//
//
// @author F. Malpartida
// ---------------------------------------------------------------------------
#include "tempCntrl.h"
#ifdef _TEMP_CONTROL_ENABLED_

#include <thermistor.h>
#include <lm75.h>

/*!
    @defined 
    @abstract   Temperature averaging samples.
    @discussion Temperature averaging sambles to adquire temperature readings
*/
#define TEMP_AVERAGE_SAMPLE     (4)

// CONSTANTS AND DEFINITIONS
// ---------------------------------------------------------------------------
/*!
 @defined    TEMP_SPEED_INC
 @abstract   Proportional speed increment around temperature control window
 @discussion Fan speed increments based on temperature control window. 
 */
#define TEMP_SPEED_INC (MAX_FAN_SPEED/(MAX_TEMP - START_TEMP))

// CLASS CONSTRUCTORS
// ---------------------------------------------------------------------------
// Constructor
tempCntrl::tempCntrl ( uint8_t fanPin, uint8_t sensorAddr )
{
   _fanPin = fanPin;
   _sensor.addressSet (sensorAddr);
   _filtTemp = 25.0;
   _speed = 175;
   
}

// PUBLIC METHODS
// ---------------------------------------------------------------------------
// Init
void tempCntrl::init ( void )
{  
   pinMode ( _fanPin, OUTPUT );
   analogWrite ( _fanPin, 150 );
   pinMode ( 13, OUTPUT );
   
   
   _sensor.init();
   
   if ( _sensor.isAvailable() == true )
   {
      _filtTemp = _sensor.readTemperature();  // Initialise the filter to the first
   }   
}

// tempGet
double tempCntrl::tempGet ( void )
{
   return ( _filtTemp );
}

// speedGet
uint8_t tempCntrl::speedGet ( void )
{
   return ( _speed );
}

// controlTemp
void tempCntrl::controlTemp ( void )
{
   double currentTemp = 0;
   
   // if the _sensor has been initialised and is available perform temperature
   // reading and filter it out
   // ------------------------------------------------------------------------
   if ( _sensor.isAvailable () )
   {
      for ( uint8_t i = 0; i < TEMP_AVERAGE_SAMPLE; i++ )
      {
         currentTemp += _sensor.readTemperature();
      }
      _filtTemp = ( FILTER_ALP * (currentTemp/TEMP_AVERAGE_SAMPLE)) + (( 1.0 - FILTER_ALP ) * _filtTemp);
   }
   
   // Set the fan speed
   fanControl ( _filtTemp );
}

// PRIVATE METHODS
// ---------------------------------------------------------------------------
void tempCntrl::fanControl ( double avrTemp )
{   
   // Temperature reading hight than maximum threshold
   if ( avrTemp > MAX_TEMP )
   {
      _speed = MAX_FAN_SPEED;
      analogWrite ( FAN_PIN, _speed );
   }
   
   // Temperature reading within control window
   else if ( avrTemp > START_TEMP )
   {
      _speed = (avrTemp - START_TEMP) * TEMP_SPEED_INC;
      
      if ( _speed < MIN_FAN_SPEED )
      {
         _speed = MIN_FAN_SPEED;
      }
      analogWrite ( FAN_PIN, _speed );
   }
   
   // Temperature below set minimum temperature, no control needed
   else
   {
      _speed = 0;
      analogWrite ( FAN_PIN, _speed );
   }
}


#endif //_TEMP_CONTROL_ENABLED_